from bs4 import BeautifulSoup, FeatureNotFound
import requests
import logging
import json

from .bandcampjson import BandcampJSON

def get_soup(url, debugging: bool = False):
    header = {'User-Agent': f'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}

    try:
        response = requests.get(url, headers=header)
    except requests.exceptions.MissingSchema:
        return None
    
    try:
        soup = BeautifulSoup(response.text, "lxml")
    except FeatureNotFound:
        soup = BeautifulSoup(response.text, "html.parser")

    return soup


def get_json(url, debugging: bool = False):

    header = {'User-Agent': f'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}

    try:
        response = requests.get(url, headers=header)
    except requests.exceptions.MissingSchema:
        return None
    
    try:
        soup = BeautifulSoup(response.text, "lxml")
    except FeatureNotFound:
        soup = BeautifulSoup(response.text, "html.parser")

    logging.debug(" Generating JSON...")

    bandcamp_json = BandcampJSON(soup, debugging).generate()
    page_json = {}
    for entry in bandcamp_json:
        page_json = {**page_json, **json.loads(entry)}

    logging.debug(' Bandcamp JSON generated...')

    return page_json

class Artist:

    def __init__(self, artist_url, debugging: bool = False):
        self.artist_title = ""
        self.artist_url = ""
        self.bio = ""
        self.profile_picture_url = ""
        self.location = ""
        self.concerts = []
        '''
        {
            "address": "",
            "venue": "",
            "date": "",
            "url": ""
        }
        '''
        self.links = []
        self.album_urls = []

        try:
            page_json = get_json(url = artist_url)

        except Exception as err:
            print(err)
            raise AttributeError("Either the Artist URL given is either private, deleted or the link is malformed")

        self.artist_title = str(page_json['artist']) # if their name is just numbers
        #self.url = page_json['byArtist']["@id"] # bc you may give a link that contains /album/.. or /track/...
        if 'track' in artist_url:
            self.artist_url = artist_url.rpartition('/track/')[0]
        elif '/album/' in artist_url:
            self.artist_url = artist_url.rpartition('/album/')[0]
        else:
            self.artist_url = artist_url

        self.artist_url = self.artist_url + '/music'

        try:
            self.bio = page_json['publisher']['description']
        except:
            self.bio = ""

        try:
            self.profile_picture_url = page_json['publisher']['image'].split('_')[0] + '_0.jpg'
        except:
            self.profile_picture_url = ""

        try:
            self.location = str(page_json['publisher']['foundingLocation']['name'])
        except:
            self.location = ""

        try:
            for concert in page_json['shows_list']:
                current_concert = {}

                try:
                    current_concert['address'] = concert['loc']
                except:
                    current_concert['address'] = ""

                try:
                    current_concert['venue'] = concert['venue']
                except:
                    current_concert['venue'] = ""

                try:
                    current_concert['date'] = int(concert['utc_date'])
                except:
                    current_concert['date'] = 0


                try:
                    current_concert['url'] = concert['uri']
                except:
                    current_concert['url'] = ""

                self.concerts.append(current_concert)
        except:
            pass


        try:
            for link in page_json['publisher']['mainEntityOfPage']:
                self.links.append(link['url'])
        except:
            pass

        soup = get_soup(self.artist_url)

        soup = soup.find('ol', {'id': "music-grid"})

        if 'track' in artist_url:
            base_link = artist_url.rpartition('/track/')[0]
        elif '/album/' in artist_url:
            base_link = artist_url.rpartition('/album/')[0]
        else:
            base_link = artist_url

        try:
            album_links = soup.find_all('a')
            for link in album_links:
                self.album_urls.append(base_link + link.get('href'))
        except:
            # there is only one album by this artist
            # thus is is just showing the album page
            # in the future, this should be an actual
            # album link, but this should be fine
            self.album_urls = [artist_url]


from datetime import datetime as dt
import json
import logging

import requests
from bs4 import BeautifulSoup
from bs4 import FeatureNotFound

from .bandcampjson import BandcampJSON
from .album import Album
from .artist import Artist
from .label import Label
from .daily import Daily, Story
from .homepage import NewAndNotable, Charts, SaleFeed, BandcampWeekly
from .genres import get_main_genres, get_subgenres
from .search import search_artists, search_albums, search_tracks, search_fans
from .track import Track

def get_json(self, url, debugging: bool = False):
    try:
        response = requests.get(url, headers=self.headers)
    except requests.exceptions.MissingSchema:
        return None

    try:
        self.soup = BeautifulSoup(response.text, "lxml")
    except FeatureNotFound:
        self.soup = BeautifulSoup(response.text, "html.parser")

    logging.debug(" Generating BandcampJSON..")
    bandcamp_json = BandcampJSON(self.soup, debugging).generate()
    page_json = {}
    for entry in bandcamp_json:
        page_json = {**page_json, **json.loads(entry)}
    logging.debug(" BandcampJSON generated..")

    return page_json

class Bandcamp:
    def __init__(self):
        self.headers = {'User-Agent': 'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}
        self.soup = None
        self.tracks = None

    def get_album(self, album_url, skip_track_scrape: bool = False):
        """Returns information for a given album URL"""
        return Album(album_url=album_url, skip_track_scrape=skip_track_scrape)

    def get_track(self, track_url):
        """Returns information for a given track URL"""
        return Track(track_url=track_url)

    def get_artist(self, artist_url):
        """Returns information for a given artist URL"""
        return Artist(artist_url=artist_url)

    def get_label(self, label_url: str):
        """Returns information for a given label URL"""
        return Label(label_url=label_url)

    def daily_latest(self, num_to_get: int):
        """Retuns the latest Bandcamp Daily stories. Warning this is very slow."""
        return Daily().search_latest(num_to_get=num_to_get)

    def daily_best_of(self,section_slug: str, num_to_get: int):
        """Returns a list of Bandcamp Daily stories for a given Best Of section"""
        return Daily().search_best_of(section_slug=section_slug, num_to_get=num_to_get)

    def daily_franchise(self, section_slug: str, num_to_get: int):
        """Returns a list of Bandcamp Daily stories for a given franchise"""
        return Daily().search_franchises(section_slug=section_slug, num_to_get=num_to_get)

    def daily_genre(self, genre_slug: str, num_to_get: int):
        """Returns a list Bandcamp Daily stories for a given genre"""
        return Daily().search_genre(genre_slug=genre_slug, num_to_get=num_to_get)

    def daily_get_story(self, story_url: str):
        """Get information on a Bandcamp Daily story"""
        return Story(story_url = story_url)

    def new_and_notable(self, num_to_get: int = 5):
        """Returns a list of new and notable releases accoding to Bandcamp"""
        return NewAndNotable(num_to_get)

    def get_genres(self):
        """Returns a list of the main overarching genres. For more specific genres see get_subgenres()"""
        # returns a list of main genres
        return get_main_genres()

    def get_subgenres(self, main_genre: str = ""):
        """Returns a list of subgenres for a given genre. If no genre is given, all are given
        :param main_genre: Slug of the main genre. See get_genre()."""
        return get_subgenres(main_genre=main_genre)

    def charts( self, main_genre: str = "", subgenre: str = "", sort: str = "", format: str = "", page: int = 0):
        """Returns a list of ChartAlbum Objects"""
        return Charts(main_genre=main_genre, subgenre=subgenre, sort=sort, format=format, page=page)

    def sales_feed(self):
        """A list of just sold entities."""
        return SaleFeed(self)

    def bandcamp_weekly(self, id: int = 0):
        """Returns information about a Bandcamp Weekly release. If no ID is given, assumes latest."""
        return BandcampWeekly.get_show(self, id=id)
    

    # doing live streams later
    #def get_live_streams(self):
        """Returns a list of ongoing Bandcamp Livestreams"""

    #def get_planned_livestreams(self):
        """Returns a list of planned Bandcamp Livestreams"""
    
    #def get_ongoing_livestreams(self):
        """Returns a list of currently boadcasting livestreams"""

    #def get_featured_livestreams(self):
        """Returns a list of Bandcamp featured livestreams"""
        
    def search_artist(self, search_string: str = ""):
        """Search Bandcamp for an artist"""
        return search_artists(search_string=search_string)
    
    def search_albums(self, search_string: str = ""):
        """Search Bandcamp for an album"""
        return search_albums(search_string=search_string)

    def search_tracks(self, search_string: str = ""):
        """Search Bandcamp for a track"""
        return search_tracks(search_string=search_string)

    def search_fans(self, search_string: str):
        """Search Bandcamp for a fan"""
        return search_fans(search_string=search_string)


import json
import requests
from bs4 import BeautifulSoup
from bs4 import FeatureNotFound
from datetime import datetime
import time

import logging

from .bandcampjson import BandcampJSON
from .track import Track

class Album:

    def __init__(self, album_url, debugging: bool = False, skip_track_scrape: bool = False):
        self.album_title = ""
        self.artist_title = ""
        self.tracks = []
        self.art_url = ""
        self.album_url = ""
        self.artist_url = ""
        self.all_songs_available = False
        self.price = {}
        self.keywords = []
        self.publisher_title = ""
        self.preorder = False

        # need to do
        self.album_bio = ""

        self.date_released = 0    # utc unixtime
        self.date_published = 0  # utc unixtime
        self.date_last_modified = 0

        # extra, ignore
        self.soup = None

        # This stuff seems *minor* so I'm just
        # going to use a dict, if this is a problem
        # for you I'm sure you're smart enough to
        # fix it or make it better
        self.advanced = {
            "copyright": "",
            "ids": {
                "artist_id": 0,
                "album_id": 0,
                "art_id": 0,
                "seller_id": 0,
            },
            "featured_track": 0,
            "email_required": False,
            "reviews": [],
            "supporters": []
        }

        try:
            page_json = self.get_json(url = album_url)

        except Exception as err:
            print(err)
            raise AttributeError("Either the album URL given is either private, deleted or the link is malformed.")

        self.artist_title = page_json['artist']
        self.artist_title = ' '.join(self.artist_title.split())

        try:
            self.album_title = page_json['current']['title']
            self.album_title = ' '.join(self.album_title.split())
        except:
            self.album_title = page_json['trackinfo'][0]['title']

        self.album_url = album_url

        # getting artist url
        if 'track' in page_json['url']:
            self.artist_url = page_json['url'].rpartition('/track/')[0]
        else:
            self.artist_url = page_json['url'].rpartition('/album')[0]

        if skip_track_scrape == False:
            for trackjson in page_json['trackinfo']:
                try:
                    track = Track(str(self.artist_url + trackjson['title_link']))
                    self.tracks.append(track)
                except:
                    pass
        else: 
            for trackjson in page_json['trackinfo']:
                try:
                    self.tracks.append(str(self.artist_url + trackjson['title_link']))
                except:
                    pass

        self.all_songs_available = self.are_all_songs_available(page_json['trackinfo'])

        self.art_url = 'https://f4.bcbits.com/img/a' + str(page_json['current']['art_id']) + '_0.jpg'

        self.keywords = page_json['keywords']

        self.publisher = page_json['publisher']['@id']

        if page_json['current']['about'] != None:
            self.album_bio = page_json['current']['about']

        # currently some albums get a time of 0, need to look if there are
        # some other places to find the released and publish times.
        try:
            self.date_released = datetime.strptime(page_json['album_release_date'], '%d %b %Y %H:%M:%S %Z')
            self.date_released = int(time.mktime(self.date_released.timetuple()))
        except:
            self.date_released = 0
        
        try:
            self.date_published = datetime.strptime(page_json['datePublished'], '%d %b %Y %H:%M:%S %Z')
            self.date_published = int(time.mktime(self.date_published.timetuple()))
        except:
            self.date_published = 0

        try:
            self.price['amount'] = float(page_json["current"]["minimum_price"])
            self.price['currency'] = page_json['albumRelease'][0]['offers']['priceCurrency']
        except:
            self.price = None

        try:
            self.preorder = bool(page_json['album_is_preorder'])
        except:
            self.preorder = False


        # doing the "advanced"
        try:
            self.advanced['copyright'] = page_json['copyrightNotice']
        except:
            self.advanced['copyright'] = ""

        try:
            self.date_last_modified = datetime.strptime(page_json['dateModified'], '%d %b %Y %H:%M:%S %Z')
            self.date_last_modified = int(time.mktime(self.date_last_modified.timetuple()))
        except:
            self.advanced['last_edited'] = self.date_published

        self.advanced['ids']['artist_id'] = page_json['current']['band_id']
        self.advanced['ids']['album_id'] = page_json['id']
        self.advanced['ids']['art_id'] = page_json['current']['art_id']
        self.advanced['ids']['seller_id'] = page_json['current']['selling_band_id']
        try:
            self.advanced['featured_track'] = page_json['additionalProperty'][1]["value"]
        except:
            self.advanced['featured_track'] = 0

        if page_json['current']['require_email'] != None:
            self.advanced['email_required'] = page_json['current']['require_email']
        else:
            self.advanced['email_required'] = False
        
        try:
            for review in page_json['comment']:
                current_review = {}
                try:
                    current_review["username"] = str(review['author']['name'])
                    current_review['username'] =  ' '.join( current_review["username"].split())
                except:
                    current_review["username"] = ""

                current_review['profile_url'] = review['author']['url']

                current_review['profile_picture'] = review['author']['image']
                current_review['profile_picture'] = current_review['profile_picture'].split('_')[0] + '_0.jpg'

                current_review['review'] = str(review['text'][0])
                current_review['review'] = ' '.join(current_review['review'].split())
                try:
                    current_review['favorite_track'] = str(review['text'][1].split(": ")[1])
                except:
                    current_review['favorite_track'] = ""

                self.advanced['reviews'].append(current_review)
        except:
            pass

        try:
            for supporter in page_json['sponsor']:
                try:
                    current_supporter = {}

                    current_supporter['username'] = supporter['name']
                    current_supporter['profile_url'] = supporter['url']
                    current_supporter['profile_picture'] = supporter['image'].split('_')[0] + '_0.jpg'

                    self.advanced['supporters'].append(current_supporter)
                except:
                    pass
        except:
            pass
    
    # other methods
    # need to organize the order of methods
    @staticmethod
    def generate_album_url(artist: str, slug: str, page_type: str) -> str:
        """Generate an album url based on the artist and album name

        :param artist: artist name
        :param slug: Slug of album/track
        :param page_type: Type of page album/track
        :return: url as str
        """
        return f"http://{artist}.bandcamp.com/{page_type}/{slug}"

    def get_album_art(self) -> str:
        """Find and retrieve album art url from page

        :return: url as str
        """
        try:
            url = self.soup.find(id='tralbumArt').find_all('a')[0]['href']
            return url
        except None:
            pass

    def get_json(self, url, debugging: bool = False):

        headers = {'User-Agent': f'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}

        try:
            response = requests.get(url, headers=headers)
        except requests.exceptions.MissingSchema:
            return None

        try:
            self.soup = BeautifulSoup(response.text, "lxml")
        except FeatureNotFound:
            self.soup = BeautifulSoup(response.text, "html.parser")

        logging.debug(" Generating BandcampJSON..")
        bandcamp_json = BandcampJSON(self.soup, debugging).generate()
        page_json = {}
        for entry in bandcamp_json:
            page_json = {**page_json, **json.loads(entry)}
        logging.debug(" BandcampJSON generated..")

        return page_json

    def are_all_songs_available(self, tracks):
        for track in tracks:
            if (track['file'] is None or False):
                return False

        return True

    def get_track_lyrics(self, track_url = None):
        track_page = requests.get(track_url, headers=None)

        try:
            track_soup = BeautifulSoup(track_page.text, 'lxml')
        except FeatureNotFound:
            track_page = BeautifulSoup(track_page.text, 'html.parser')
        try:
            track_lyrics = track_soup.find('div', {'class': 'lyricsText'})
        except:
            return ''

        if track_lyrics:
            return track_lyrics.text
        else:
            return ''


import requests
from bs4 import BeautifulSoup, FeatureNotFound
import json
import random
import logging
from .bandcampjson import BandcampJSON
from datetime import datetime
import time


def get_json(url, debugging: bool = False):

    header = {'User-Agent': f'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}

    try:
        response = requests.get(url, headers=header)
    except requests.exceptions.MissingSchema:
        return None
    
    try:
        soup = BeautifulSoup(response.text, "lxml")
    except FeatureNotFound:
        soup = BeautifulSoup(response.text, "html.parser")

    logging.debug(" Generating JSON...")

    bandcamp_json = BandcampJSON(soup, debugging).generate()
    page_json = {}
    for entry in bandcamp_json:
        page_json = {**page_json, **json.loads(entry)}

    logging.debug(' Bandcamp JSON generated...')

    return page_json

def get_current_new_and_notable_id():
    ping_size = 7          # how many guesses should be do per get/ping, max 60
    check_width = 7        # how many future IDs should be checked if they exist
    highest_known = 1835    # the highest number that got a response
    lowest_unknown = 999999      # the lowest seen number that got a response
    latest_id = None

    still_looking = True

    while still_looking:

        currently_guessing = []

        try:
            currently_guessing = random.sample(range(highest_known, lowest_unknown), ping_size)
        except:
            currently_guessing = random.sample(range(highest_known - check_width, lowest_unknown + check_width), ping_size)
        

        # get information
        id_string = ''
        for id in currently_guessing:
            id_string += str(id) + ','
        r = requests.get('https://bandcamp.com/api/notabletralbum/2/get?id=' + id_string)

        # see what becomes valid and invalid
        results = r.json()

        # for each guess, get next check_width entry ids
        for id in currently_guessing:
            # make check
            check_string = ''
            for i in range(check_width - 1):
                check_string += str(id + i) + ','

            check = requests.get('https://bandcamp.com/api/notabletralbum/2/get?id=' + check_string)
            check = check.json()
            
            # if a single one returns then it is the new highest known
            if check != {}:
                # it is not empty
                if len(check) == 1:
                    still_looking = False
                    latest_id = id
                    break
                if id > highest_known:
                    highest_known = id
            else:
                # it is empty
                if id < lowest_unknown:
                    lowest_unknown = id

    return latest_id

class NewAndNotable():

    def __init__(self, num_to_get: int = 5):
        self.albums = []

        current_id = get_current_new_and_notable_id()
        #ids = []
        id_string = ''

        for i in range(num_to_get):
            id_string += (str(current_id - i)) + ','

        entries = requests.get('https://bandcamp.com/api/notabletralbum/2/get?id=' + id_string)
        entries = entries.json()

        for key in entries:
            album = NewAndNotableAlbum()
            entry = {}
            entry = entries[str(key)]

            album.album_artist = entry['artist']

            album.album_title = entry['title']

            album.album_art = "https://f4.bcbits.com/img/a" + str(entry['art_id']) + "_0"

            album.album_url = entry['tralbum_url']

            album.genre = entry['genre']

            album.date_published = datetime.strptime(entry['published_date'], '%d %b %Y %H:%M:%S %Z')
            album.date_published = int(time.mktime(album.date_published.timetuple()))

            album.date_last_modified = datetime.strptime(entry['mod_date'], '%d %b %Y %H:%M:%S %Z')
            album.date_last_modified = int(time.mktime(album.date_last_modified.timetuple()))

            album.description = entry['desc']

            album.advanced['band_id'] = entry['band_id']
            album.advanced['album_id'] = entry['tralbum_id']
            album.advanced['art_id'] = entry['art_id']
            album.advanced['featured_track_id'] = entry['featured_track_id']

            self.albums.append(album)

class NewAndNotableAlbum():
    def __init__(self):

        self.album_artist = ""
        self.album_title = ""
        self.album_art = ""
        self.album_url = ""
        self.genre = ""
        self.date_published = 0
        self.date_last_modified = 0
        self.description = ""
        self.advanced = {}

        
class Charts():

    def __init__(self, 
                main_genre: str = "all", 
                sort: str = "top", 
                page: int = 0, 
                format: str = 'all', 
                subgenre: str = ""):

        self.albums = []

        if not main_genre:
            main_genre = "all"

        if not sort:
            sort = 'top'

        if page == None or not page:
            page = 0

        if not format:
            format = 'all'

        if not subgenre:
            subgenre = ""

        arg_string = ""

        arg_string += 'g=' + main_genre

        arg_string += '&s=' + sort

        if (main_genre) != 'all' and ('rec' not in sort):
            # can contain subgenre
            arg_string += '&t=' + subgenre

        if 'rec' not in sort:
            # can contain format
            arg_string += '&f=' + format
        else:
            # there is a artist rec filter
            # well which one
            # most or latest
            if 'most' in sort:
                arg_string += '&r=most'
            else:
                arg_string += '&r=latest'

        arg_string += '&p=' + str(page)
        
        r = requests.get('https://bandcamp.com/api/discover/3/get_web?' + arg_string)
        r = r.json()

        for current_album in r['items']:
            album = ChartsAlbum()

            album.album_artist = current_album['secondary_text']

            album.album_title = current_album['primary_text']

            if current_album['url_hints']['custom_domain'] not in ['null', 'Null', 'None']:
                album.artist_url = 'https://' + current_album['url_hints']['subdomain'] + '.bandcamp.com'
            else:
                album.artist_url = current_album['url_hints']['custom_domain']

            if current_album['url_hints']['item_type'] == 'a':
                album.album_url = album.artist_url + '/album/' + current_album['url_hints']['slug']
            else:
                album.album_url = album.artist_url + '/track/' + current_album['url_hints']['slug']

            album.genre = current_album['genre_text']

            if current_album['location_text'] not in ['null', 'Null', "none", "None"] and current_album['location_text'] is not None:
                album.artist_location = current_album['location_text']
            else:
                album.artist_location = ""

            album.date_published = datetime.strptime(current_album['publish_date'], '%d %b %Y %H:%M:%S %Z')
            album.date_published = int(time.mktime(album.date_published.timetuple()))

            album.featured_track = {
                "title": current_album['featured_track']['title'],
                "id": current_album['featured_track']['id'],
                "duration": current_album['featured_track']['duration']
            }

            album.advanced = {
                "band_id": current_album['band_id'],
                "album_id": current_album['id'],
                "art_id": current_album['art_id'],
                "bio_image_id": current_album['bio_image']['image_id']
            }

            self.albums.append(album)

class ChartsAlbum():

    def __init__(self):
        self.album_artist = ""
        self.album_title = ""
        self.artist_url = ""
        self.album_url = ""
        self.genre = ""
        self.artist_location = ""
        self.date_published = 0
        self.featured_track = {}
        self.advanced = {}

class SaleFeed():

    def __init__(self, time: int = 0):

        self.sold_time = 0.00000
        #self.artist = ""
        self.purchases = []
        if time == 0:
            # looks like they didnt give a start time
            # assume they just want the whatever the latest is
            r = requests.get('https://bandcamp.com/api/salesfeed/1/get')
        else:
            r = requests.get('https://bandcamp.com/api/salesfeed/1/get?start_date=' + str(time))

        r = r.json()
        for sale in r['events']:

            bought_items = []

            self.sold_time = sale['utc_date']

            for current_item in sale['items']:
                # because you can buy multiple items
                item = Item()

                item.price = {
                    "currency": current_item['currency'],
                    "price": current_item['item_price'],
                    "paid": current_item['amount_paid']
                }

                item.description = current_item['item_description']

                item.url = 'https:' + current_item['url']

                if current_item['slug_type'] == 'a':
                    item.type = 'album'
                elif current_item['slug_type'] == 't':
                    item.type = 'track'
                elif current_item['slug_type'] == 'b':
                    item.type = 'discography'
                else:
                    item.type = 'physical'

                if item.type == 'physical':
                    item.image_url = "https://f4.bcbits.com/img/" + str(current_item['package_image_id']) +"_0.jpg"
                else:
                    item.image_url = "https://f4.bcbits.com/img/a" + str(current_item['art_id']) +"_0.jpg"
                bought_items.append(item)

            self.purchases.append(bought_items)

        
class Item():
    def __init__(self):
        self.price = {}
            # paid
            # price
            # currency
        self.image_url = ""

        self.description = ""

        self.type = ""
            # t for track
            # p seems to be for physical
            # a for album
            # b is full disco

        self.url = ""

class BandcampWeekly():

    def __init__(self):

        # time
        self.date_released = 0
        self.date_published = 0

        # image
        self.image_url = ""

        # audio
        self.stream_url = ""
        self.duration_seconds = 0.00

        # episode
        self.episode_title = ""
        self.series_title = "" # Normal BCW or the hipho show
        self.episode_description_long = ""
        self.episode_description_short = ""
        self.episode_subtitle = ""
        self.episode_id = 0

        # tracks
        self.tracks = []

    def get_show(self, id: int):
        if id == 0:
            r = requests.get('https://bandcamp.com/api/bcweekly/3/list')
            r = r.json()
            id = r['results'][0]['id']
        
        r = requests.get("https://bandcamp.com/api/bcweekly/2/get?id=" + str(id))
        r = r.json()

        self.date_released = datetime.strptime(r['date'], '%d %b %Y %H:%M:%S %Z')
        self.date_released = int(time.mktime(self.date_released.timetuple()))

        self.date_published = datetime.strptime(r['published_date'], '%d %b %Y %H:%M:%S %Z')
        self.date_published = int(time.mktime(self.date_published.timetuple()))

        self.image_url = "https://f4.bcbits.com/img/" + str(r['show_image_id']) + '_0.jpg'

        self.stream_url = r['audio_stream']['mp3-128']

        self.duration_seconds = r['audio_duration']

        self.episode_title = r['audio_title']
        self.series_title = r['title']
        self.episode_description_long = r['desc']
        self.episode_description_short = r['short_desc']
        self.episode_subtitle = r['subtitle']
        self.episode_id = r['show_id']
        self.tracks = []

        for track in r['tracks']:
            track_object = BandcampWeeklyTrack()

            if track['label'] != None:
                track_object.label_title = track['label']

            try:
                track_object.art_url = "https://f4.bcbits.com/img/a" + str(track['track_art_id']) + "_0.jpg"
            except:
                pass
            
            try:
                if track['album_title'] != None:
                    track_object.album_title = track['album_title']
                else:
                    track_object.album_title = track['title']
            except:
                pass

            track_object.price = {
                "currency": track['currency'],
                "amount": track['price']
            }
            if '/album/' in track['url']:
                track_object.artist_url = track['url'].split('/album/')[0]
                track_object.album_title = track['album_title']
            else:
                track_object.artist_url = track['url'].split('/track/')[0]

            try:
                track_object.album_url = track['album_url']
            except:
                pass

            track_object.track_url = track['track_url']
            track_object.track_title = track['title']
            track_object.artist_title = track['artist']

            try:
                track_object.location = track['location_text']
            except:
                pass
            
            track_object.start_timecode = track['timecode']

            self.tracks.append(track_object)

        return self

class BandcampWeeklyTrack():

    def __init__(self):
        self.art_url = ""
        self.album_title = ""
        self.label_title = ""
        self.price = {}
            # "currency": "USD",
            # "amount": 0.00
        self.artist_url = ""
        self.album_url = ""
        self.track_url = ""
        self.track_title = ""
        self.artist_title = ""
        self.location = ""
        self.start_timecode = 0



import requests
from bs4 import BeautifulSoup, FeatureNotFound

class Label:

    def __init__(self, label_url: str, debugging: bool = False):

        self.label_title = ""
        self.location = ""
        self.bio = ""
        self.links = []
        self.bands = []
        '''
        {
            "name": "",
            "location": "",
            "picture": "",
            "url": ""
        }
        '''
        self.albums = []
        """
        {
            "artist": "",
            "title": "",
            "art_url": "",
            "url": ""
        }
        """
        self.merch_url = ""
        self.subscription_url = ""
        self.community_url = ""         # maybe in the future, scrape the page
        self.profile_picture_url = ""

        header = {'User-Agent': 'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}

        #change the url to bands
        label_url = label_url.split('/')
        label_url = label_url[0] + '//' + label_url[2]  # [1] is empty
        artists_url = label_url + '/artists'
        albums_url = label_url + '/music'

        try:
            response = requests.get(url=artists_url, headers=header)
        except requests.exceptions.MissingSchema:
            return None
        
        try:
            soup = BeautifulSoup(response.text, "lxml")
        except FeatureNotFound:
            soup = BeautifulSoup(response.text, "html.parser")

        name_location = soup.find('p', {"id": "band-name-location"})
        self.label_title = name_location.find('span', {"class": "title"}).get_text()

        self.location = name_location.find("span", {"class": "location secondaryText"}).get_text()

        self.bio = soup.find("p", {"id": "bio-text"}).get_text()

        links = soup.find("ol", {"id": "band-links"})
        links = links.find_all("a")
        for link in links:
            self.links.append(link.get('href'))

        artists = soup.find("div", {"class": "leftMiddleColumns"})
        artists = artists.find_all("li")

        for artist in artists:
            band = {}
            band['name'] = artist.text
            band['name'] = ' '.join(band['name'].split())

            band['location'] = artist.find("div", {"class": "artists-grid-location secondaryText"}).get_text()
            band['location'] = ' '.join(band['location'].split())
            try:
                band["picture"] = artist.find("img").get('data-original').split('_')[0] + '_0.jpg'
            except:
                try:
                    band["picture"] = artist.find("img").get('src').split('_')[0] + '_0.jpg'
                except:
                    band["picture"] = ""

            band["url"] = artist.find("a").get('href')

            self.bands.append(band)

        # changing url to the album url
        try:
            response = requests.get(url=albums_url, headers=header)
        except requests.exceptions.MissingSchema:
            return None
        
        try:
            soup = BeautifulSoup(response.text, "html5lib")
        except FeatureNotFound:
            soup = BeautifulSoup(response.text, "html.parser")

        albums = soup.find("ol", {"id": "music-grid"})
        albums = albums.find_all("li")

        for album in albums:

            current_album = {}

            try:
                current_album["artist"] = album.find('span', {"class": "artist-override"}).text
                current_album['artist'] = ' '.join(current_album['artist'].split())
            except:
                current_album['artist'] = ''


            current_album['title'] = album.find("p", {"class": "title"}).get_text().replace(current_album['artist'], "")
            current_album['title'] = ' '.join(current_album['title'].split())

            try:
                current_album['art_url'] = album.find("img").get('data-original').split('_')[0] + '_0.jpg'
            except:
                current_album['art_url'] = album.find("img").get('src').split('_')[0] + '_0.jpg'

            #current_album['art_url'] = album.find("img").get("src")

            if 'http' not in album.find('a').get('href'):
                current_album['url'] = label_url + album.find('a').get('href')
            else:
                current_album['url'] = album.find('a').get('href')

            self.albums.append(current_album)

        self.merch_url = label_url + '/merch'
        self.subscription_url = label_url + '/fan-club'
        self.community_url = label_url + '/community'

        self.profile_picture_url = soup.find('div', {'id': "rightColumn"})
        self.profile_picture_url = self.profile_picture_url.find('div', {"id": "bio-container"})


import requests
from bs4 import BeautifulSoup, FeatureNotFound
import logging
import json
from .bandcampjson import BandcampJSON
from datetime import datetime
import time

def string_clean(input_string):
    return ' '.join(input_string.split())

class Daily():
    def __init__(Bandcamp):
       """Bandcamp Daily"""

    def search_latest(self, num_to_get: int):
        """Gets the latests stories form the Bandcamp Daily
        :param num_to_get: The number of BCDaily posts to return. Will allways return an array of article links."""
        page = 1
        url = "https://daily.bandcamp.com/latest?page="
        posts_obtained = []
        header = {'User-Agent': 'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}
        while True:
            try:
                response = requests.get(url=url + str(page), headers=header)
            except requests.exceptions.MissingSchema as Err:
                return Err
            
            try:
                soup = BeautifulSoup(response.text, "lxml")
            except FeatureNotFound:
                soup = BeautifulSoup(response.text, "html.parser")

            articles = soup.find("articles-list")
            try:
                articles = articles.find_all('div', {"class": "title-wrapper"})
                #list = []

                for article in articles:
                    if len(posts_obtained) < num_to_get:
                        posts_obtained.append("https://daily.bandcamp.com" + article.find('a').get('href'))
                    else:
                        return posts_obtained
            except:
                return posts_obtained

            page += 1

    def search_best_of(self, section_slug: str, num_to_get: int):
        """Seach for BCDaily posts in the Best Of section.
        :param section_slug: The URL slug of the section you want. Example for Best of 2016: 'best-of-2016'
        :param num_to_get: The number of BCDaily posts to return. Will allways return an array of article links."""
        header = {'User-Agent': 'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}
        url = "https://daily.bandcamp.com/"
        page = 1
        posts_obtained = []

        while True:
            try:
                response = requests.get(url=url + section_slug  + '?page=' + str(page), headers=header)
            except requests.exceptions.MissingSchema:
                return ""
            
            try:
                soup = BeautifulSoup(response.text, "lxml")
            except FeatureNotFound:
                soup = BeautifulSoup(response.text, "html.parser")

            articles = soup.find("articles-list")
            try:
                articles = articles.find_all('div', {"class": "title-wrapper"})

                for article in articles:
                    if len(posts_obtained) < num_to_get:
                        posts_obtained.append("https://daily.bandcamp.com" + article.find('a').get('href'))
                    else:
                        return posts_obtained
            except:
                return posts_obtained

            page += 1
            print("Going to page " + str(page))
                



    def search_franchises(self, section_slug: str, num_to_get: int):
        """Seach the BCDaily in the Franchises section.
        :param section_slug: The URL slug of the section you want. Example for Album Of The Day: album-of-the-day
        :param num_to_get: The number of BCDaily posts to return. Will allways return an array of article links."""
        header = {'User-Agent': 'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}
        url = "https://daily.bandcamp.com/"
        page = 1
        posts_obtained = []

        while True:
            try:
                response = requests.get(url=url + section_slug  + '?page=' + str(page), headers=header)
            except requests.exceptions.MissingSchema:
                return ""
            
            try:
                soup = BeautifulSoup(response.text, "lxml")
            except FeatureNotFound:
                soup = BeautifulSoup(response.text, "html.parser")

            articles = soup.find("articles-list")
            try:
                articles = articles.find_all('div', {"class": "title-wrapper"})

                for article in articles:
                    if len(posts_obtained) < num_to_get:
                        posts_obtained.append("https://daily.bandcamp.com" + article.find('a').get('href'))
                    else:
                        return posts_obtained
            except:
                return posts_obtained

            page += 1

    def search_genre(self, genre_slug: str, num_to_get: int):
        """Search the BCDail in the Genre section.
        :param genre_slug: The URL slug of the genre you want. Example for R&B/Soul: r-b-soul
        :param num_to_get: The number of BCDaily posts to return. Will allways return an array of article links."""
        header = {'User-Agent': 'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}
        url = "https://daily.bandcamp.com/genres/"
        page = 1
        posts_obtained = []

        while True:
            try:
                response = requests.get(url=url + genre_slug  + '?page=' + str(page), headers=header)
            except requests.exceptions.MissingSchema:
                return ""
            
            try:
                soup = BeautifulSoup(response.text, "lxml")
            except FeatureNotFound:
                soup = BeautifulSoup(response.text, "html.parser")

            try:
                articles = soup.find("articles-list")
                articles = articles.find_all('div', {"class": "title-wrapper"})
                list = []

                for article in articles:
                    if len(posts_obtained) < num_to_get:
                        posts_obtained.append("https://daily.bandcamp.com/genres" + article.find('a').get('href'))
                    else:
                        return posts_obtained
            except:
                return posts_obtained

            page += 1        

def get_json(url, debugging: bool = False):

    header = {'User-Agent': f'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}

    try:
        response = requests.get(url, headers=header)
    except requests.exceptions.MissingSchema:
        return ""
    
    try:
        soup = BeautifulSoup(response.text, "lxml")
    except FeatureNotFound:
        soup = BeautifulSoup(response.text, "html.parser")

    logging.debug(" Generating JSON...")

    bandcamp_json = BandcampJSON(soup, debugging).generate()
    page_json = {}
    for entry in bandcamp_json:
        page_json = {**page_json, **json.loads(entry)}

    logging.debug(' Bandcamp JSON generated...')

    return page_json

class Story():
    def __init__(self, story_url: str):
        self.title = ""
        self.author = ""
        self.description = ""
        self.headline = ""
        self.main_image_url = ""
        self.story_type = ""
        self.linked_in_text = []
        self.linked_media = []
        self.text = ""
        self.date_published = 0
        self.date_last_modified = 0
        self.images = []        # look for story with multiple images
        self.tags = []

        response = requests.get(story_url)

        try:
            page_json = get_json(url = story_url)
        except Exception as err:
            print(err)
            raise AttributeError("Either the Artist URL gicen is either private, deleted or the link is malformed")
        
        try:
            soup = BeautifulSoup(response.text, "lxml")
        except FeatureNotFound:
            soup = BeautifulSoup(response.text, "html.parser")
        
        try:
            soup = BeautifulSoup(response.text, "lxml")
        except FeatureNotFound:
            soup = BeautifulSoup(response.text, "html5lib")

        self.title = string_clean(soup.find('article-title').text)

        self.author = soup.find('article-credits')
        self.author = string_clean(self.author.find('a').text)

        self.story_type = string_clean(soup.find('article-type').text.lower())
        body = soup.find('article')
        body = body.find_all('p')

        for section in body:
            for link in section.find_all('a'):
                #self.linked_in_text.append(link.absolute_links.pop())
                self.linked_in_text.append(link.get('href'))
            if string_clean(section.text):
                self.text = self.text + string_clean(section.text) + '\n\n'                

        players = soup.find_all("div", {"class": "player-wrapper"})


        '''
        I wanted to make a thing
        to get songs that were featured
        in a BCD story, but a large amount
        of the information is rendered
        in JS, I tried to get it with
        requests-html and its render
        funtion but could not get it
        to work propery. Sorry.
        '''

        try:
            self.date_published = datetime.strptime(page_json['datePublished'], '%Y-%m-%dT%H:%M:%SZ')
            self.date_published = int(time.mktime(self.date_published.timetuple()))
        except:
            self.published = 0

        try:
            self.date_last_modified = datetime.strptime(page_json['dateModified'], '%Y-%m-%dT%H:%M:%SZ')
            self.date_last_modified = int(time.mktime(self.advanced['last_edited'].timetuple()))
        except:
            self.date_last_modified = self.date_published

        self.description = page_json['description']

        self.headline = page_json['headline']

        self.main_image_url = page_json['image']

        '''
        bandcamp does not seem to have a standard way
        to deal with images
        sometimes its in a gallery class
        sometimes its in an inline class
        sometimes its just laying there

        so i am loading the entire article body
        then deleting all music, artist and merch
        that is all on the side
        '''
        body = soup.find('article')

        artists = body.find_all("mplayer-artist")       # artists
        for artist in artists:
            artist.decompose()
        players = body.find_all('mplayer')              # players
        for player in players:
            player.decompose()
        all_merch = body.find_all('mplayer-sidebar')    # merch
        for merch in all_merch:
            merch.decompose()

        for photo in body.find_all('img'):
            if photo.get('src') is not None:
                self.images.append(photo.get('src'))

        self.tags = page_json['keywords'].split(', ')



'''
Need to do
'''
from .bandcampjson import BandcampJSON
import requests
import logging
from bs4 import BeautifulSoup, FeatureNotFound
import json
from datetime import datetime
import time

def are_all_songs_available(tracks):
        for track in tracks:
            if (track['file'] is None or False):
                return False

        return True

def get_track_lyrics(track_url = None):
    track_page = requests.get(track_url, headers=None)

    try:
        track_soup = BeautifulSoup(track_page.text, 'lxml')
    except FeatureNotFound:
        track_page = BeautifulSoup(track_page.text, 'html.parser')
    try:
        track_lyrics = track_soup.find('div', {'class': 'lyricsText'})
    except:
        return ''

    if track_lyrics:
        return track_lyrics.text
    else:
        return ''

def get_json(url, debugging: bool = False):

    header = {'User-Agent': f'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}

    try:
        response = requests.get(url, headers=header)
    except requests.exceptions.MissingSchema:
        return None
    
    try:
        soup = BeautifulSoup(response.text, "lxml")
    except FeatureNotFound:
        soup = BeautifulSoup(response.text, "html.parser")

    logging.debug(" Generating JSON...")

    bandcamp_json = BandcampJSON(soup, debugging).generate()
    page_json = {}
    for entry in bandcamp_json:
        page_json = {**page_json, **json.loads(entry)}

    logging.debug(' Bandcamp JSON generated...')

    return page_json

class Track:
    def __init__(self, track_url):
        self.track_title = ""                 #
        self.artist_title = ""                #
        self.album_title = ""                 #
        self.track_number = 0           #
        self.date_published = 0              #
        self.date_released = 0               #
        self.date_last_modified = 0          #
        self.tags = []                  #
        self.price = {}                 #
            # {
            # "currency": "USD",
            # "amount": 0.00
            # }
        self.album_art_url = ""
        self.duration_seconds = 0.000   #
        self.track_url = ""             #
        self.album_url = ""             #
        self.artist_url = ""            #
        self.available = False          #
        self.lyrics = ""                #

        try:
            page_json = get_json(url = track_url)

        except Exception as err:
            print(err)
            raise AttributeError("Either the album URL given is either private, deleted or the link is malformed.")

        for track_json in page_json['trackinfo']:
            # i know its its a list of 1 element, dont care
            self.track_title = track_json['title']
            self.track_number = track_json['track_num']
            self.duration_seconds = track_json['duration']

            self.artist_title = page_json['byArtist']['name']

            try:
                self.artist_url = page_json['byArtist']['@id']
            except:
                # ther is no artist url, so its link is the publisher link
                self.artist_url = page_json['publisher']['@id']

            try:
                self.album_title = page_json['inAlbum']['name']
                self.album_url = page_json['inAlbum']['@id']
            except:
                # its a single
                self.album_title = ""
                self.album_url = page_json['url']

            self.track_url = track_url



            try:
                self.lyrics = get_track_lyrics(track_url)
            except:
                self.lyrics = ''

            if track_json['file'] is None:
                self.available = False
            else:
                self.available = True

            try:
                self.date_published = datetime.strptime(page_json['datePublished'], '%d %b %Y %H:%M:%S %Z')
                self.date_published = int(time.mktime(self.date_published.timetuple()))
            except:
                self.date_published = 0

            try:
                self.date_released = datetime.strptime(page_json['album_release_date'], '%d %b %Y %H:%M:%S %Z')
                self.date_released = int(time.mktime(self.date_released.timetuple()))
            except:
                self.date_released = 0

            try:
                self.date_last_modified = datetime.strptime(page_json['dateModified'], '%d %b %Y %H:%M:%S %Z')
                self.date_last_modified = int(time.mktime(self.date_last_modified.timetuple()))
            except:
                self.date_released = 0

            for tag in page_json['keywords']:
                self.tags.append(tag)

            try:
                self.price['currency'] = page_json['inAlbum']['albumRelease'][1]['offers']['priceCurrency']
                self.price['amount'] = page_json['inAlbum']['albumRelease'][1]['offers']['price']
            except:
                self.price['currency'] = page_json['inAlbum']['albumRelease'][0]['offers']['priceCurrency']
                self.price['amount'] = page_json['inAlbum']['albumRelease'][0]['offers']['price']

            self.album_art_url = page_json['image'].split('_')[0] + '_0.jpg'


import requests
from bs4 import BeautifulSoup
from datetime import datetime
import time

def clean_string(string):
    return ' '.join(string.split())

def search_artists(search_string: str):
    link = "https://bandcamp.com/search?q=" + search_string + '&item_type=b'
    try:
        response = requests.get(link)
    except requests.exceptions.MissingSchema as err:
        return err

    try:
        soup = BeautifulSoup(response.tex, "lxml")
    except:
        soup = BeautifulSoup(response.text, "html.parser")

    results = soup.find('ul', {"class": "result-items"})
    things_to_return = []

    for item in results.find_all("li"):
        band = ArtistResults()
        band.artist_title = ' '.join(item.find('div', {"class": "heading"}).text.split())
        
        try:
            band.location = ' '.join(item.find('div', {"class": "subhead"}).text.split())
        except:
            band.location = ""

        try:
            band.genre = ' '.join(item.find("div", {"class": "genre"}).text.split())
        except:
            band.genre = ''

        try:
            band.tags = ' '.join(item.find('div', {'class': "tags data-search"}).text.split())
            band.tags = band.tags.split(": ")[1:]
            band.tags = ' '.join(band.tags).split(', ')
        except:
            band.tags = []

        try:
            band.image_url = item.find('img').get('src').split('_')[0]
            band.image_url = band.image_url + '_0.jpg'
        except:
            band.image = ""

        band.artist_url = item.find('div', {'class': "itemurl"})
        band.artist_url = band.artist_url.find('a').text
        things_to_return.append(band)

    return things_to_return

def search_albums(search_string: str):
    link = "https://bandcamp.com/search?q=" + search_string + '&item_type=a'
    try:
        response = requests.get(link)
    except requests.exceptions.MissingSchema as err:
        return err

    try:
        soup = BeautifulSoup(response.tex, "lxml")
    except:
        soup = BeautifulSoup(response.text, "html.parser")

    results = soup.find('ul', {"class": "result-items"})
    things_to_return = []

    for item in results.find_all("li"):
        album = AlbumResults()

        album.album_title = clean_string(item.find('div', {"class": "heading"}).text)

        album.artist_title = clean_string(item.find('div', {"class": "subhead"}).text[3:])

        album.total_tracks = clean_string(item.find('div', {"class": "length"}).text)
        album.total_tracks = int(''.join(album.total_tracks).split(' track')[0])

        album.duration_seconds = int(item.find('div', {"class": 'length'}).text.split(', ')[1].split( )[0])*60

        album.date_released = ' '.join(item.find('div', {'class': "released"}).text.split())[9:]
        album.date_released = datetime.strptime(album.date_released, '%d %B %Y')
        album.date_released = int(time.mktime(album.date_released.timetuple()))

        album.album_url = clean_string(item.find('div', {"class": "itemurl"}).text)

        album.tags = ' '.join(item.find('div', {'class': "tags data-search"}).text.split())
        album.tags = album.tags.split(": ")[1:]
        album.tags = ' '.join(album.tags).split(', ')

        album.album_art_url = item.find('div', {"class": "art"})
        album.album_art_url = album.album_art_url.find('img').get('src')[:-5] + '0.jpg'
        
        things_to_return.append(album)

    return things_to_return


def search_tracks(search_string: str):
    link = "https://bandcamp.com/search?q=" + search_string + '&item_type=t'
    try:
        response = requests.get(link)
    except requests.exceptions.MissingSchema:
        None

    try:
        soup = BeautifulSoup(response.tex, "lxml")
    except:
        soup = BeautifulSoup(response.text, "html.parser")

    results = soup.find('ul', {"class": "result-items"})
    things_to_return = []

    for item in results.find_all("li"):
        track = TrackResults()

        track.track_title = clean_string(item.find('div', {"class": "heading"}).text)

        track.album_title = clean_string(item.find('div', {"class": "subhead"}).text).split(' by ')[0].replace('from ', '')
        
        track.artist_title = clean_string(item.find('div', {'class': "subhead"}).text).split('by ')[1]

        track.date_released = ' '.join(item.find('div', {'class': "released"}).text.split())[9:]
        track.date_released = datetime.strptime(track.date_released, '%d %B %Y')
        track.date_released = int(time.mktime(track.date_released.timetuple()))

        track.album_art_url = item.find('img').get('src').split('_')[0] +'_0.jpg'

        track.track_url = clean_string(item.find('div', {'class': "itemurl"}).text)
        
        things_to_return.append(track)

    return things_to_return

def search_fans(search_string: str):
    link = "https://bandcamp.com/search?q=" + search_string + '&item_type=f'
    try:
        response = requests.get(link)
    except requests.exceptions.MissingSchema:
        None

    try:
        soup = BeautifulSoup(response.tex, "lxml")
    except:
        soup = BeautifulSoup(response.text, "html.parser")

    results = soup.find('ul', {"class": "result-items"})
    things_to_return = []

    for item in results.find_all("li"):
        fan = FanResults()

        fan.username = clean_string(item.find("div", {"class": "heading"}).text)
        fan.fan_url = clean_string(item.find('div', {"class": "itemurl"}).text)

        try:
            fan.genre = clean_string(item.find('div', {"class": "genre"}).text).split(': ')[1]
        except:
            fan.genre = ""

        try:
            fan.profile_picture_url = clean_string(item.find('img').get('src')).split('_')[0]+'_0.jpg'
        except:
            fan.profile_picture_url = ""

        things_to_return.append(fan)

    return things_to_return


class ArtistResults():

    def __init__(self):
        self.artist_title = ""
        self.location = ""
        self.genre = ""
        self.tags = []
        self.image_url = ""
        self.artist_url = ""

class AlbumResults():

    def __init__(self):
        self.album_title = ""
        self.artist_title = ""
        self.total_tracks = 0
        self.duration_seconds = 0
        self.date_released = 0
        self.album_url = ""
        self.tags = []
        self.album_art_url = ""

class TrackResults():

    def __init__(self):
        self.track_title = ""
        self.album_title = ""
        self.artist_title = ""
        self.date_released = 0
        self.album_art_url = ""
        self.track_url = ""

class FanResults():

    def __init__(self):
        self.username = ""
        self.fan_url = ""
        self.genre = ""
        self.profile_picture_url = ""


from .bandcampjson import BandcampJSON
from bs4 import BeautifulSoup, FeatureNotFound
import requests
import logging
import json

def get_json(url, debugging: bool = False):

        headers = {'User-Agent': f'bandcamp-api/0 (https://github.com/RustyRin/bandcamp-api)'}

        try:
            response = requests.get(url, headers=headers)
        except requests.exceptions.MissingSchema:
            return None

        try:
            soup = BeautifulSoup(response.text, "lxml")
        except FeatureNotFound:
            soup = BeautifulSoup(response.text, "html.parser")

        logging.debug(" Generating BandcampJSON..")
        bandcamp_json = BandcampJSON(soup, debugging).generate()
        page_json = {}
        for entry in bandcamp_json:
            page_json = {**page_json, **json.loads(entry)}
        logging.debug(" BandcampJSON generated..")

        return page_json

def get_main_genres():
    try:
        page_json = get_json(url="https://intlanthem.bandcamp.com/album/in-these-times")
    except Exception as err:
        print("Error getting the sample album for the genres")
        print(err)
        raise AttributeError

    genres = []
    for genre_dict in page_json['signup_params']['genres']:
        genres.append(genre_dict['value'])

    return genres

def get_subgenres(main_genre: str):

    if main_genre == "" or main_genre == None:
        raise Exception('Main genre slug was not provided')
    
    try:
        page_json = get_json(url="https://intlanthem.bandcamp.com/album/in-these-times")
    except Exception as err:
        print("Error getting the sample album for the genres")
        print(err)
        raise AttributeError

    subgenres = []

    for subgenre in  page_json['signup_params']['subgenres'][main_genre]:
        subgenres.append(subgenre['value'])

    return subgenres


from setuptools import *

setup(
    name="bandcamp_api",
    version="0.1.14",
    description="Obtains information from bandcamp.com",
    author="RustyRin",
    packages=['bandcamp_api'],
    url="https://github.com/RustyRin/bandcamp-api/",
    install_requires=["setuptools", "bs4", "demjson3", 'html5lib', 'lxml'],
    keywords=["api", "bandcamp"],
    zip_safe=False
    )